//////////////////////////////////////////////////////////////////////NVMH3////
// File:  $Id: //sw/devrel/SDK/MEDIA/scripts/rtzImport.cs#1 $
// 
// Copyright NVIDIA Corporation 2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// Comments:
// 	Import FX files that have been exported by RTZen Ginza.
//		Ginza files are also DXSAS file, but have a few Ginza-specific semantics.
//		This script makes a copy of the Ginza shader with those semantics converted
//			to FX-Composer-friendly terms.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Xml;
using System.Collections;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using interop.nv_sys;
using System.Runtime.InteropServices;

namespace PluginList
{
	public class RtzImport :INVUtility {

		// Report an error from the script
		private void ReportError(INVSystem Sys, string Text)
		{
			NVReport Rep = new NVReportClass();
			Rep.SetReport(eNVREPORT.NVREPORT_ERROR, Text, "", -1);
			Sys.Report(Rep);
		}

		// Report a log from the script
		private void Report(INVSystem Sys, string Text)
		{
			NVReport Rep = new NVReportClass();
			Rep.SetReport(eNVREPORT.NVREPORT_LOG, Text, "", -1);
			Sys.Report(Rep);
		}

		public string GetText(INVString str)
		{
			string strOut;
			str.GetString(out strOut);
			return strOut;
		}

		// make an NVstring from a string
		NVString CreateNVString(string In)
		{
			NVString str = new NVString();
			str.SetString(In);
			return str;
		}

		// get a list of FX files
		public string[] GetFiles()
		{
    		OpenFileDialog openFileDialog1 = new OpenFileDialog();
		    openFileDialog1.InitialDirectory = "" ;
		    openFileDialog1.FileName="";
    		openFileDialog1.Filter = ".fx files(*.fx)|*.fx|All files (*.*)|*.*" ;
    		openFileDialog1.FilterIndex = 1 ;
    		openFileDialog1.RestoreDirectory = false ;
    		openFileDialog1.Multiselect = true;
    		openFileDialog1.Title = "Select RTZen-Generated .FX Files";
    		if(openFileDialog1.ShowDialog() == DialogResult.OK)
    		{
    			return openFileDialog1.FileNames;
    		}
    		return null;
    	}

		private void composerize_file(string srcFileName,string dstFileName)
		{
			FileInfo fxSource = new FileInfo(srcFileName);
			StreamReader source = fxSource.OpenText();
			StreamWriter dest = new StreamWriter(dstFileName,false);
			dest.WriteLine("//\n// Imported by rtzImport.cs\n//\n");
			string text;
			do 
			{
				text = source.ReadLine();
				if (text == null) break;
				text = Regex.Replace(text,@"(:\sView|World|Projection)+IT;",
											"$1InverseTranspose;");
				text = Regex.Replace(text,@"(:\sView|World|Projection)+I;",
											"$1Inverse;");
				text = Regex.Replace(text,@"(:\sView|World|Projection)+T;",
											"$1Transpose;");
				text = Regex.Replace(text,@"string name ",
											"string resourcename ");
				text = Regex.Replace(text,@"(float4x4\s+\w+\s*:\s*\w+);",
											"$1 <string UIWidget=\"none\";>;");
				text = Regex.Replace(text,@"(:\s*Time\s)",
											"$1 <string UIWidget=\"none\";> ");
				text = Regex.Replace(text,@"float4\s+(\w+)\s*:\s*WorldEyePosition",
											"// $0\n"+
											"float4x4 VI_$1 : ViewInverse <string UIWidget=\"none\";>;\n"+
											"static float4 $1 = float4(VI_$1[3].xyz,1)");
				text = Regex.Replace(text,@"string\s+UIType\s*=",
					"string UIWidget =");
				text = Regex.Replace(text,@"NoiseTable2D.*",
					"$0 // Not in FX Composer");
				Regex nt = new Regex(@"texture\s+(?<var>\w+)\s*:\s*NoiseTable2D\s*;");
				MatchCollection nMatches = nt.Matches(text);
				foreach (Match mt in nMatches) {
					if (mt.Length != 0) 
					{
						text = "// "+text+"\n";
						string var = mt.Groups["var"].ToString();
						string func = var+"_noise_2d";
						string scMacro = var.ToUpper() + "_SCALE";
						string szMacro = var.ToUpper() + "_SIZE";
						string fmMacro = var.ToUpper() + "_FORMAT";
						text = text + "#define "+scMacro+" 1\n";
						text = text + "#define "+szMacro+" 128\n";
						text = text + "#define "+fmMacro+" \"A8B8G8R8\"\n";
						text = text +
"float4 "+func+@"(float2 Pos : POSITION) : COLOR {
    float4 Noise = (float4)0;
    for (int i = 1; i < 256; i += i) {
        Noise.r += (noise(Pos * "+scMacro+@" * i)) / i;
        Noise.g += (noise((Pos + 1)* "+scMacro+@" * i)) / i;
        Noise.b += (noise((Pos + 2) * "+scMacro+@" * i)) / i;
        Noise.a += (noise((Pos + 3) * "+scMacro+@" * i)) / i;
    }
    return (Noise + 0.5);
}
texture "+var+"  <\n" +
						"    string TextureType = \"2D\";\n" +
						"    string function = \""+func+"\";\n"+
						"    string Format = ("+fmMacro+");\n"+
						"    string UIWidget = \"None\";\n"+
						"    int width = "+szMacro+", height = "+szMacro+";\n>;\n";
					}
				}
				Regex lp = new Regex(@"LightParameters\s+(?<var>\w+)\[(?<count>\d+)\]\s*:\s*WorldLightParameters;.*");
				MatchCollection matches = lp.Matches(text);
				foreach (Match mt in matches) {
					if (mt.Length != 0) 
					{
						text = "// "+text+"\n";
						string cs = mt.Groups["count"].ToString();
						int count = Int32.Parse(cs);
						for (int i=0; i<count; ++i) 
						{
							text = text + string.Format("float4 LightPos{0} "+
								"<string Object = \"pointlight{0}\"; > = ",i) +
								"{1,1,0,1};\n";
						}
						for (int i=0; i<count; ++i) 
						{
								text = text + string.Format("float4 LightCol{0}" +
											"<string Object = \"pointlight{0}\"; " +
											"string UIWidget = \"color\";> = ",i) +
											"{1,1,1,0};\n";
						}
						text = text + "static LightParameters "+
										mt.Groups["var"]+
										"[" + count.ToString() + "]" +
										" = {\n";
						for (int i=0; i<count; ++i) 
						{
							text = text + string.Format("\tLightPos{0}, LightCol{0}",i);
							if (i != (count-1)) 
							{
								text = text + ",\n";
							} 
							else 
							{
								text = text + "};";
							}
						}
					}
				}
				dest.WriteLine(text);
				// Console.WriteLine(text);
			} while (text != null);
			source.Close();
			dest.Close();
		}

		private string new_fx(string OldFx)
		{
			string newFx = Regex.Replace(OldFx,@"\.fx$","_fxcomp.fx");
			return newFx;
		}

		public int Run(interop.nv_sys.NVSystem Sys)
		{
			NVScene Scene;
			Sys.GetScene(out Scene);
			NVMaterial Mat;
			string[] effects = GetFiles();
			if (effects == null)
				return 0;
			foreach(string effect in effects)
			{
				string newFileName = new_fx(effect);
				composerize_file(effect,newFileName);
				NVString MatName;
				Sys.GenerateUniqueName(CreateNVString("material"),
										CreateNVString(newFileName),
										out MatName);
				Scene.CreateMaterial(MatName, CreateNVString(newFileName), out Mat);
			}

			return 1;
		}
	}
}

///////////////////////////// eof ///
